package ink.wgink.service.dao;

import ink.wgink.pojo.dtos.ZTreeDTO;
import ink.wgink.pojo.dtos.menu.MenuDTO;
import org.springframework.stereotype.Repository;

import java.util.List;
import java.util.Map;

/**
 * @ClassName: IMenuDao
 * @Description: 菜单
 * @Author: wenc
 * @Date: 2018/12/27 10:40 PM
 * @Version: 1.0
 **/
@Repository
public interface IMenuDao {

    /**
     * 菜单列表
     *
     * @param params
     * @return
     */
    List<MenuDTO> listMenus(Map<String, Object> params);

    /**
     * 菜单详情
     *
     * @param params
     * @return
     * @throws SearchException
     */
    MenuDTO getMenu(Map<String, Object> params);

    /**
     * zTree菜单列表
     *
     * @param params
     * @return
     */
    List<ZTreeDTO> listZTreeMenus(Map<String, Object> params);

    /**
     * 子节点数量
     *
     * @param getPId
     * @return
     */
    Integer countByParentId(String getPId);

    /**
     * 通过父ID查询最后一条子菜单
     *
     * @param parentId
     * @return
     */
    MenuDTO getLastByParentId(String parentId);

    /**
     * 新增菜单
     *
     * @param params
     */
    void saveMenu(Map<String, Object> params);

    /**
     * 删除菜单
     *
     * @param params
     */
    void removeMenu(Map<String, Object> params);

    /**
     * 修改菜单
     *
     * @param params
     */
    void updateMenu(Map<String, Object> params);

    /**
     * 通过用户获取菜单ID
     *
     * @param params
     * @return
     */
    List<String> listMenuIdByUser(Map<String, Object> params);
}
