package ink.wgink.service.service.impl;

import com.github.pagehelper.PageHelper;
import com.github.pagehelper.PageInfo;
import ink.wgink.common.base.DefaultBaseService;
import ink.wgink.common.component.SecurityComponent;
import ink.wgink.exceptions.SearchException;
import ink.wgink.pojo.ListPage;
import ink.wgink.pojo.dtos.ZTreeDTO;
import ink.wgink.pojo.dtos.menu.MenuDTO;
import ink.wgink.pojo.result.SuccessResult;
import ink.wgink.pojo.result.SuccessResultList;
import ink.wgink.service.dao.IMenuDao;
import ink.wgink.service.service.IMenuService;
import ink.wgink.util.UUIDUtil;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;
import java.util.Map;

/**
 * @ClassName: MenuServiceImpl
 * @Description: 菜单
 * @Author: wenc
 * @Date: 2018/12/27 10:18 PM
 * @Version: 1.0
 **/
@Service
public class MenuServiceImpl extends DefaultBaseService implements IMenuService {

    @Autowired
    private IMenuDao menuDao;
//    @Autowired
//    private IRoleDao roleDao;
//    @Autowired
//    private IOauthClientService oauthClientService;
//    @Autowired
//    private SecurityComponent securityComponent;
//    @Autowired
//    private IUserService userService;

    @Override
    public List<MenuDTO> listMenuAllJson(Map<String, Object> params) {
        if (!SecurityComponent.USERNAME_ADMIN.equals(securityComponent.getCurrentUsername())) {
            LOG.debug("非管理员菜单查询");
            // params.put("menuIds", listUserMenuId());
        }
        return listMenusAll(params);
    }

    @Override
    public List<MenuDTO> listMenusAll(Map<String, Object> params) {
        List<MenuDTO> menuDTOs = new ArrayList<>(menuDao.listMenus(params));
        // listSubMenus(menuDTOs, params);
        return menuDTOs;
    }

    @Override
    public MenuDTO getMenu(Map<String, Object> params) {
        return menuDao.getMenu(params);
    }

    @Override
    public List<ZTreeDTO> listZTreeMenus(Map<String, Object> params) {
        List<ZTreeDTO> zTreeDTOs = menuDao.listZTreeMenus(params);
        for (ZTreeDTO zTreeDTO : zTreeDTOs) {
            Integer subCount = menuDao.countByParentId(zTreeDTO.getId());
            setZTreeInfo(zTreeDTO, subCount);
        }
        return zTreeDTOs;
    }

    @Override
    public SuccessResultList<List<MenuDTO>> listPageMenus(ListPage page) {
        PageHelper.startPage(page.getPage(), page.getRows());
        List<MenuDTO> menuDTOs = new ArrayList<>(menuDao.listMenus(page.getParams()));
        PageInfo<MenuDTO> pageInfo = new PageInfo<>(menuDTOs);
        return new SuccessResultList<>(menuDTOs, pageInfo.getPageNum(), pageInfo.getTotal());
    }

    @Override
    public synchronized SuccessResult saveMenu(Map<String, Object> params) {
        String parentCode = null;
        String menuParentId = params.get("menuParentId").toString();
        if (!"0".equals(menuParentId)) {
            params.put("menuId", menuParentId);
            MenuDTO menuDTO = menuDao.getMenu(params);
            parentCode = menuDTO.getMenuCode();
        }
        String menuCode = getCode(parentCode, menuParentId);
        params.put("menuCode", menuCode);
        params.put("menuId", UUIDUtil.getUUID());
        setSaveInfo(params);
        menuDao.saveMenu(params);
        return new SuccessResult();
    }

    @Override
    public SuccessResult removeMenu(Map<String, Object> params) {
        params.put("menuIds", Arrays.asList(params.get("menuIds").toString().split("_")));
        setUpdateInfo(params);
        menuDao.removeMenu(params);
        return new SuccessResult();
    }

//    @Override
//    public SuccessResultData<List<MenuDTO>> listMenuByClientId(Map<String, Object> params) {
//        OauthClientDTO oauthClientDTO = oauthClientService.getOauthClient(params);
//        if (oauthClientDTO == null) {
//            throw new SearchException("客户端不存在");
//        }
//        String menuId = oauthClientDTO.getMenuId();
//        if (StringUtils.isBlank(menuId)) {
//            return new SuccessResultData<>(new ArrayList<>());
//        }
//        // 获取全部菜单
//        params.clear();
//        params.put("menuParentId", menuId);
//        params.put("menuStatus", 0);
//        if (!SecurityComponent.USERNAME_ADMIN.equals(securityComponent.getCurrentUsername())) {
//            log.debug("非管理员第三方查询菜单");
//            params.put("menuIds", listUserMenuId());
//        }
//        List<MenuDTO> menuDTOs = listMenusAll(params);
//        return new SuccessResultData<>(menuDTOs);
//    }

//    @Override
//    public List<String> listUserMenuId() {
//        List<RoleMenuBO> roleMenuBOs = listRoleMenu();
//        List<String> menuIds = new ArrayList<>();
//        for (RoleMenuBO roleMenuBO : roleMenuBOs) {
//            menuIds.add(roleMenuBO.getMenuId());
//        }
//        return menuIds;
//    }

//    @Override
//    public SuccessResultData<List<MenuDTO>> listMenuByClientIdAndUserId(String clientId, String userId) {
//        Map<String, Object> params = new HashMap<>(1);
//        params.put("clientId", clientId);
//        OauthClientDTO oauthClientDTO = oauthClientService.getOauthClient(params);
//        if (oauthClientDTO == null) {
//            throw new SearchException("客户端不存在");
//        }
//        String menuId = oauthClientDTO.getMenuId();
//        if (StringUtils.isBlank(menuId)) {
//            return new SuccessResultData<>(new ArrayList<>());
//        }
//        List<String> userMenuIds;
//        // 如果是管理员获取管理员配置的菜单，管理员的ID是1
//        if (StringUtils.equalsIgnoreCase(userId, "1")) {
//            userMenuIds = listAdminMenuId();
//        } else {
//            params.put("userId", userId);
//            params.put(IRoleService.ROLE_TYPE, IRoleService.ROLE_MENU);
//            userMenuIds = listMenuIdByUser(params);
//        }
//        // 获取全部菜单
//        params.clear();
//        params.put("menuParentId", menuId);
//        params.put("menuStatus", 0);
//        params.put("menuIds", userMenuIds);
//        List<MenuDTO> menuDTOs = listMenusAll(params);
//        return new SuccessResultData<>(menuDTOs);
//    }

    @Override
    public List<String> listMenuIdByUser(Map<String, Object> params) {
        return menuDao.listMenuIdByUser(params);
    }

    @Override
    public SuccessResult updateMenu(Map<String, Object> params) {
        setUpdateInfo(params);
        menuDao.updateMenu(params);
        return new SuccessResult();
    }

    /**
     * 获取角色菜单列表
     *
     * @return
     * @throws SearchException
     */
//    private List<RoleMenuBO> listRoleMenu() {
//        Map<String, Object> params = new HashMap<>(2);
//        List<String> listRoleIds = securityComponent.listRoleIds();
//        params.put("listRoleIds", listRoleIds);
//        params.put("roleType", IRoleService.ROLE_MENU);
//        List<RoleMenuBO> roleMenuBOs = roleDao.listRoleMenuInfo(params);
//        return roleMenuBOs;
//    }

    /**
     * 管理员菜单列表
     *
     * @return
     * @throws SearchException
     */
//    private List<RoleMenuBO> listAdminMenu() {
//        Map<String, Object> params = new HashMap<>(4);
//        params.put("listRoleIds", Arrays.asList(SecurityComponent.USERNAME_ADMIN));
//        params.put("roleType", IRoleService.ROLE_MENU);
//        List<RoleMenuBO> roleMenuBOs = roleDao.listRoleMenuInfo(params);
//        return roleMenuBOs;
//    }

    /**
     * 管理员菜单ID列表
     *
     * @return
     */
//    private List<String> listAdminMenuId() {
//        List<RoleMenuBO> roleMenuBOs = listAdminMenu();
//        List<String> menuIds = new ArrayList<>();
//        for (RoleMenuBO roleMenuBO : roleMenuBOs) {
//            menuIds.add(roleMenuBO.getMenuId());
//        }
//        return menuIds;
//    }

    /**
     * 递归查询子菜单
     *
     * @param menuDTOs
     * @param params
     */
//    private void listSubMenus(List<MenuDTO> menuDTOs, Map<String, Object> params) {
//        if (!StringUtils.isBlank(params.get("roleId") == null ? null : params.get("roleId").toString())) {
//            String type = params.get("type").toString();
//            if (IRoleService.AUTH_TYPE_INSERT.equals(type)) {
//                params.put(IRoleService.ROLE_TYPE, IRoleService.ROLE_INSERT);
//            } else if (IRoleService.AUTH_TYPE_DELETE.equals(type)) {
//                params.put(IRoleService.ROLE_TYPE, IRoleService.ROLE_DELETE);
//            } else if (IRoleService.AUTH_TYPE_UPDATE.equals(type)) {
//                params.put(IRoleService.ROLE_TYPE, IRoleService.ROLE_UPDATE);
//            } else if (IRoleService.AUTH_TYPE_QUERY.equals(type)) {
//                params.put(IRoleService.ROLE_TYPE, IRoleService.ROLE_QUERY);
//            } else {
//                params.put(IRoleService.ROLE_TYPE, IRoleService.ROLE_MENU);
//            }
//            log.debug("管理员菜单查询");
//            if (StringUtils.equals(SecurityComponent.USERNAME_ADMIN, params.get("roleId").toString())) {
//                List<RoleMenuBO> roleMenuBOs = listAdminMenu();
//                if (!Objects.isNull(roleMenuBOs) && !roleMenuBOs.isEmpty()) {
//                    List<String> menuIds = new ArrayList<>();
//                    for (RoleMenuBO roleMenuBO : roleMenuBOs) {
//                        menuIds.add(roleMenuBO.getMenuId());
//                    }
//                }
//                listSubMenus(menuDTOs, params, roleMenuBOs);
//            } else {
//                List<RoleMenuBO> roleMenuBOs = roleDao.listRoleMenuInfo(params);
//                listSubMenus(menuDTOs, params, roleMenuBOs);
//            }
//        } else {
//            listSubMenus(menuDTOs, params, null);
//        }
//    }

    /**
     * 子菜单权限
     *
     * @param menuDTOs
     * @param params
     * @param roleMenuBOs
     * @throws SearchException
     */
//    private void listSubMenus(List<MenuDTO> menuDTOs, Map<String, Object> params, List<RoleMenuBO> roleMenuBOs) {
//        for (MenuDTO menuDTO : menuDTOs) {
//            if (roleMenuBOs != null) {
//                boolean hasRight = false;
//                for (RoleMenuBO roleMenuBO : roleMenuBOs) {
//                    if (menuDTO.getMenuId().equals(roleMenuBO.getMenuId())) {
//                        hasRight = true;
//                        break;
//                    }
//                }
//                menuDTO.setRight(hasRight);
//            }
//            params.put("menuParentId", menuDTO.getMenuId());
//            List<MenuDTO> subMenuDTOs = new ArrayList<>(menuDao.listMenus(params));
//            menuDTO.setSubMenus(subMenuDTOs);
//            if (!subMenuDTOs.isEmpty()) {
//                menuDTO.setParent(true);
//            } else {
//                menuDTO.setParent(false);
//            }
//            listSubMenus(subMenuDTOs, params, roleMenuBOs);
//        }
//    }

    /**
     * 获取code
     *
     * @param parentCode
     * @param parentId
     * @return
     * @throws SearchException
     */
    private String getCode(String parentCode, String parentId) {
        MenuDTO menuDTO = menuDao.getLastByParentId(parentId);
        String code = menuDTO != null ? menuDTO.getMenuCode() : "0000";
        return super.getNewCode(code, parentCode);
    }

}
